package main

import (
	"flag"
	"fmt"
	"github.com/gorilla/mux"
	"gitto.work/shortcut/enhancedfileserver"
	"log"
	"net/http"
)

type config struct {
	listen   string
	port     int
	username string
	password string
	noindex  bool
	log      bool
}

var c config

func init() {
	flag.StringVar(&c.listen, "listen", "0.0.0.0", "IP address to listen.")
	flag.IntVar(&c.port, "port", 8000, "Listen port for interface (ports below 1024 may require super user privileges)")
	flag.StringVar(&c.username, "username", "", "Require this username tp access")
	flag.StringVar(&c.password, "password", "", "Require this password to access interface")
	flag.BoolVar(&c.noindex, "noindex", false, "Disable directory indexing")
	flag.BoolVar(&c.log, "log", false, "Enable simple request log")
	flag.Parse()
}

func main() {
	r := mux.NewRouter()
	if c.username != "" && c.password != "" {
		r.Use(BasicAuth)
	}
	if c.log {
		r.Use(QueryLog)
	}

	if c.noindex {
		r.PathPrefix("/").Handler(http.StripPrefix("/", enhancedfileserver.FileServerNoIndex(http.Dir("./"))))
	} else {
		r.PathPrefix("/").Handler(http.StripPrefix("/", enhancedfileserver.FileServer(http.Dir("./"))))
	}

	a := fmt.Sprintf("%s:%d", c.listen, c.port)

	log.Fatal(http.ListenAndServe(a, r))
}

func BasicAuth(next http.Handler) http.Handler {
	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		if c.username != "" && c.password != "" {
			user, pass, ok := r.BasicAuth()
			if !ok || user != c.username || pass != c.password {
				w.Header().Set("WWW-Authenticate", `Basic realm="mydomains"`)

				http.Error(w, "Unauthorized", http.StatusUnauthorized)
			}
		}

		next.ServeHTTP(w, r)

	})
}

func QueryLog(next http.Handler) http.Handler {
	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		log.Printf("%s - %s", r.RemoteAddr, r.URL)
		next.ServeHTTP(w, r)
	})
}
